//------------------------------------------------------------------------------
// Lamp : Open source game middleware
// Copyright (C) 2004  Junpei Ohtani ( Email : junpee@users.sourceforge.jp )
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

/** @file
 * シーンヘッダ
 * @author Junpee
 */

#ifndef SCENE_H_
#define SCENE_H_

#include <Graphics/System/GraphicsDeviceObjectHolder.h>
#include <Core/Container/ArrayList.h>

namespace Lamp{

class Renamer;
class DrawRequest;
class SceneNode;
class Camera;
class Fog;
class Mesh;

class CameraManager;
class SceneNodeManager;
class LightManager;
class ModelManager;
class MeshManager;
class MeshDataManager;
class MaterialManager;
class TextureManager;
class PictureManager;

//------------------------------------------------------------------------------
/**
 * シーン
 */
class Scene : public GraphicsDeviceObjectHolder{
friend class LampGraphics;
public:
	/**
	 * クリア
	 *
	 * 強制的にシーン内の全オブジェクトを破棄します
	 * @return 削除したオブジェクト数
	 */
	virtual int clear();

	//--------------------------------------------------------------------------
	/**
	 * 走査
	 */
	virtual void traverse();

	//--------------------------------------------------------------------------
	/**
	 * メッシュリストの取得
	 * @param meshList メッシュリスト
	 * @param camera 刈り込むカメラ
	 */
	virtual void getMeshList(ArrayList<Mesh*>* meshList, Camera* camera);

	/**
	 * ローカルライトリストの取得
	 * @param mesh メッシュ
	 * @param drawRequest 描画リクエスト
	 */
	virtual void getLocalLightList(Mesh* mesh, DrawRequest* drawRequest);

	//--------------------------------------------------------------------------
	/**
	 * 名前の取得
	 * @return 名前
	 */
	virtual const String& getName() const{ return name_; }

	//--------------------------------------------------------------------------
	/**
	 * ルートノードの取得
	 * @return ルートノード
	 */
	virtual SceneNode* getRootNode(){ return rootNode_; }

	//--------------------------------------------------------------------------
	/**
	 * カレントカメラの設定
	 * @param currentCamera カレントカメラ
	 */
	virtual void setCurrentCamera(Camera* currentCamera){
		currentCamera_ = currentCamera;
	}

	/**
	 * カレントカメラの取得
	 * @return カレントカメラ
	 */
	virtual Camera* getCurrentCamera(){ return currentCamera_; }

	//--------------------------------------------------------------------------
	/**
	 * フォグの取得
	 * @return フォグ
	 */
	virtual Fog* getFog(){ return fog_; }

	//--------------------------------------------------------------------------
	/**
	 * チックの設定
	 * @param tick チック
	 */
	virtual void setTick(int tick){ tick_ = tick; }

	/**
	 * チックの取得
	 * @return チック
	 */
	virtual int getTick() const{ return tick_; }

	//--------------------------------------------------------------------------
	// リネーム関係
	//--------------------------------------------------------------------------
	/**
	 * リネーマの設定
	 * @param renamer 設定するリネーマ
	 */
	virtual void setRenamer(Renamer* renamer);

	/**
	 * リネーマの取得
	 * @return リネーマ
	 */
	virtual Renamer* getRenamer() const{ return renamer_; }

	//--------------------------------------------------------------------------
	/**
	 * デバイスオブジェクトの初期化
	 * @return 成功したらtrueを返す
	 */
	virtual bool initializeGraphicsDeviceObjects();

	/**
	 * デバイスオブジェクトの削除
	 */
	virtual void deleteGraphicsDeviceObjects();

	/**
	 * デバイスオブジェクトのリストア
	 * @return 成功したらtrueを返す
	 */
	virtual bool restoreGraphicsDeviceObjects();

	/**
	 * デバイスオブジェクトの無効化
	 */
	virtual void invalidateGraphicsDeviceObjects();

	//--------------------------------------------------------------------------
	/**
	 * カメラマネージャの取得
	 * @return カメラマネージャ
	 */
	virtual CameraManager* getCameraManager(){ return cameraManager_; }

	/**
	 * シーンノードマネージャの取得
	 * @return シーンノードマネージャ
	 */
	virtual SceneNodeManager* getSceneNodeManager(){
		return	sceneNodeManager_;
	}

	/**
	 * ライトマネージャの取得
	 * @return ライトマネージャ
	 */
	virtual LightManager* getLightManager(){ return lightManager_; }

	/**
	 * モデルマネージャの取得
	 * @return モデルマネージャ
	 */
	virtual ModelManager* getModelManager(){ return modelManager_; }

	/**
	 * メッシュマネージャの取得
	 * @return メッシュマネージャ
	 */
	virtual MeshManager* getMeshManager(){ return meshManager_; }

	/**
	 * メッシュデータマネージャの取得
	 * @return メッシュデータマネージャ
	 */
	virtual MeshDataManager* getMeshDataManager(){ return meshDataManager_; }

	/**
	 * マテリアルマネージャの取得
	 * @return マテリアルマネージャ
	 */
	virtual MaterialManager* getMaterialManager(){ return materialManager_; }

	/**
	 * テクスチャマネージャの取得
	 * @return テクスチャマネージャ
	 */
	virtual TextureManager* getTextureManager(){ return textureManager_; }

	/**
	 * ピクチャマネージャの取得
	 * @return ピクチャマネージャ
	 */
	virtual PictureManager* getPictureManager(){ return pictureManager_; }

	//--------------------------------------------------------------------------
protected:
	/**
	 * コンストラクタ
	 * @param name 名前
	 */
	Scene(const String& name);

	/**
	 * デストラクタ
	 */
	virtual ~Scene();

	//--------------------------------------------------------------------------
private:
	// コピーコンストラクタの隠蔽
	Scene(const Scene& copy);

	// 代入コピーの隠蔽
	void operator =(const Scene& copy);

	// 名前
	String name_;
	// リネーマ
	Renamer* renamer_;
	// ルートノード
	SceneNode* rootNode_;
	// カレントカメラ
	Camera* currentCamera_;
	// フォグ
	Fog* fog_;
	// カメラマネージャ
	CameraManager* cameraManager_;
	// シーンノードマネージャ
	SceneNodeManager* sceneNodeManager_;
	// ライトマネージャ
	LightManager* lightManager_;
	// モデルマネージャ
	ModelManager* modelManager_;
	// メッシュマネージャ
	MeshManager* meshManager_;
	// メッシュデータマネージャ
	MeshDataManager* meshDataManager_;
	// マテリアルマネージャ
	MaterialManager* materialManager_;
	// テクスチャマネージャ
	TextureManager* textureManager_;
	// ピクチャマネージャ
	PictureManager* pictureManager_;
	// チック
	int tick_;
};

//------------------------------------------------------------------------------
} // End of namespace Lamp
#endif // End of SCENE_H_
//------------------------------------------------------------------------------
